# -*- coding: utf-8 -*-


def comp_height(self):
    """Compute the height of the magnet

    Parameters
    ----------
    self : MagnetType13
        A MagnetType13 object

    Returns
    -------
    Hmag: float
        The magnet's height [m]

    """

    return self.Hmag
