# -*- coding: utf-8 -*-

pixmap_dict = {
    "End_Random": ":/images/images/MachineSetup/WindParam/End_Random.PNG",
    "End_Preform": ":/images/images/MachineSetup/WindParam/End_Preform.PNG",
    "Dim_Ext_Rotor": ":/images/images/MachineSetup/MachineDimension/Dim_ER.png",
    "Dim_In_Rotor_No_Shaft": ":/images/images/MachineSetup/MachineDimension/Dim_IRNS.png",
    "Dim_In_Rotor_Shaft": ":/images/images/MachineSetup/MachineDimension/Dim_IRS.png",
    "WindingCW2LT": ":/images/images/MachineSetup/WindingPattern/Type_Wind_1.PNG",
    "WindingCW1L": ":/images/images/MachineSetup/WindingPattern/Type_Wind_2.PNG",
    "WindingDW2L": ":/images/images/MachineSetup/WindingPattern/Type_Wind_3.PNG",
    "WindingDW1L": ":/images/images/MachineSetup/WindingPattern/Type_Wind_4.PNG",
    "WindingCW2LR": ":/images/images/MachineSetup/WindingPattern/Type_Wind_5.PNG",
    "WindingKoil": ":/images/images/MachineSetup/WindingPattern/Type_Wind_7.png",
    "WindingMat": ":/images/images/MachineSetup/WindingPattern/Type_Wind_7.png",
    "SCIM": ":/images/images/MachineSetup/SCIM.png",
    "DFIM": ":/images/images/MachineSetup/DFIM.png",
    "SyRM": ":/images/images/MachineSetup/SyRM.png",
    "SPMSM": ":/images/images/MachineSetup/SPMSM.png",
    "IPMSM": ":/images/images/MachineSetup/IPMSM.png",
    "BPMSM": ":/images/images/MachineSetup/BPMSM.png",
    "WRSM": ":/images/images/MachineSetup/WRSM.png",
}
