# -*- coding: utf-8 -*-


def get_cell_area(self, indices=None):
    """Return the area of the cells on the outer surface.

    Parameters
    ----------
    self : MeshMat
        a MeshMat object
    indices : list
        list of the points to extract (optional)

    Returns
    -------
    areas: ndarray
        Area of the cells
    """

    # TODO

    return area
